' h2incz.bas [Copyright (C) 2012, sarbayo]
' using:  Bxbasic v20.3.1   --   Win32 executable
'
' The original h2inc program is designed to translate GCC C header files 
' into ".inc" include files, more or less usable by Masm and other 
' assemblers.
' This program functions similar to h2inc, with a number of exception.
'
' Presently, the primary goal of this program is to extract the #define 
' and extern statements and produce clean and uncluttered variable 
' definitions and function prototypes and not to attempt to parse the 
' conditional #if statements, typically found in header files.
'
' As stated, conditional statements are ignored, entirely. 
' Instead, each line is parsed and tested for a valid and understandable 
' variable definition that can be translated into an assembly language 
' equate statement.
'
' How to run this program:
' ========================
' 1) First, by default, this program is designed to read the contents of a 
' text file, (named header.txt), in the working directory, which contains 
' a list of the header file names to be translated. 
' The header.txt file must be formated in a specific way. 
' The first line of the file must contain the number of headers to be 
' processed. The second and subsequent lines must contain the names of the 
' header files, without the ".h" extension and must appear between quotes, 
' like so:
'                          __________
'                         |10
'                         |"a.out"
'                         |"aio"
'                         |"alloca"
'                         |"ar"
'                         |"argp"
'                         |"argz"
'                         |...etc
'
'
' 2) Copies of the headers to be translated must be placed in a sub-
' directory of the current working directory, named "Linux-Headers". 
' As an example, assume the working directory is named h2incz somewhere on 
' the current drive: i.e.: 
'      c:\..\h2incz
'
' the files required to reside in the working directory would be: 
'  bxbasic.exe, 
'  h2incz.bas and 
'  headers.txt.
'
' 3) Next, create a subdirectory named: "Linux-Headers"
'      c:\..\h2incz\Linux-Headers
'
' Store copies of the gcc headers to be processed into that subdirectory. 
'
' 4) Create a second subdirectory and name it: "Linux-Inc".
' This is where the translated files will be stored.
'      c:\..\h2incz\Linux-Inc
'
' 5) To run, from the command-line, type: 
'      bxbasic h2incz
'
'
' Anywhere from 1 to 100 (or more) header files may be processed this way.
'
' The resulting output files will be usable, but, not in their entirety in 
' the outputted format. Many items, will be commented-out, with the 
' exception of function prototypes. Includes, defines and possibly other 
' things will be commented-out. Since the main objective of this program 
' is to provide usable function prototypes, that is where the main focus of 
' the output files will be placed. 
'
' Those items which are commented-out can be un-commented and tested 
' individually. The reason for this is that many define statements are 
' subject to conditional statements and in some cases do not apply in a 
' given context.
' 
' It should be understood, (by the user), that GCC (as do most C compilers)
' uses a preprocessor to read, parse and interpret the C header files. This
' small program is in no way a preprecessor and falls far short of 
' interpreting the meaning and context of the statements contained in the 
' header files. This simple program simply strips-out and dumps what 
' information may be useful to the Assembly Language programmer.
'
' **NOTE:
' The prototypes and equate statements generated are purely theoretical.
' Most of the files generated by this program are untested. There are just
' too many header files contained in the GCC LIBC to be able to test them 
' all in the time I have available. 
' Should you find errors (and I am sure there will be several), in the 
' output files, please let me know, so that I can make corrections to the 
' program. 
' Thanks, Steve.
'
'
' List of things ToDo:
' Conditional statements, struct's, union's, typedef's, and macro's.
' See documentation at the end of this file for additional explainations 
' and examples.
'
'
' Program begins here:
'==========================================================================
'         DECLARE SUB FUNCTIONS:
'==========================================================================
'
    DECLARE SUB GetAlpha(chr$)
    DECLARE SUB IsWhitespace(ndx)
    DECLARE SUB StripDefine()
    DECLARE SUB GetAlNum(chr$)
    DECLARE SUB MakeEquate()
    DECLARE SUB VerifyInput()
    DECLARE SUB StripComments()
    DECLARE SUB ExternType(ndx)
    DECLARE SUB StripExtern()
    DECLARE SUB getVarName(ndx, len)
    DECLARE SUB SkipNonAlnum(pi)
    DECLARE SUB CompExtern(ndx)
    DECLARE SUB BuildInc()
    DECLARE SUB getVarName2(ndx, len)
    DECLARE SUB SortConst(pi, len)
    DECLARE SUB fixNCursesExport()
    DECLARE SUB fixStruct()
    DECLARE SUB skipVarName(ndx, lenz)
    DECLARE SUB fixConstChar()
    DECLARE SUB fixConstChar2()
    DECLARE SUB TrailingWhitespace(indx, xstrng$)
    DECLARE SUB fixUnsigned()
    DECLARE SUB fixFuncPntr(pi)
    DECLARE SUB WalkBackward(indx)
    DECLARE SUB fetchVarName(ptr)
    DECLARE SUB MakeQuotedStr()
    DECLARE SUB getQuotedStr(ndx, lenz)
    DECLARE SUB StripTypecasts(new$)
    DECLARE SUB fixEnum()
    DECLARE SUB GetPath(chr$)
    DECLARE SUB getPathFilename(ndx, lenz)

'==========================================================================
'         LOAD HEADER NAMES:
'==========================================================================
'
    DIM finname$ = "", hname$ = ""
    DIM indx = 0, arraycount = 0
'                                    load header names into array:
    OPEN "I", #1, "headers.txt"
    INPUT #1, arraycount
    Print "array count="; arraycount
    DIM namearray$(arraycount)

    FOR indx = 1 TO arraycount
        INPUT #1, hname$
        namearray$(indx) = hname$
' Print namearray$(indx)

    NEXT indx
    Close 1
'
'==========================================================================
'         DIM GLOBAL VARIABLES:
'==========================================================================
'
    DIM inext$ = ".h"
    DIM outext$ = ".inc"
    DIM foutname$ = "", finname$ = ""
    DIM fin$ = "", fout$ = ""
'                                     list input path here:
    DIM inpath$ = "Linux-Headers\"
'                                     list output path here:
    DIM outpath$ = "Linux-Inc\"
'                                     create temp file-names here:
    DIM tmpfile1$ = "tempfile1.tmp"
    DIM tmpfile2$ = "tempfile2.tmp"
    DIM tmpfile3$ = "tempfile3.tmp"
    DIM tmpfile4$ = "tempfile4.tmp"
'
'                                     declare GLOBAL VARIABLES here:
    DIM input$ = "", rstr$ = ""
    DIM test1$ = "", ch$ = "", define1$ = "#define"
    DIM define2$ = "# defin", extern$ = "extern"
    DIM define3$ = "#  ", define4$ = "define", include$ = "#include"
    DIM len1 = 0, ndx = 0, rval = 0, pi = 0
'
'==========================================================================
'         HEADERS PROCESSING LOOP:
'==========================================================================

    CLS
    FOR indx = 1 TO arraycount
        finname$ = namearray$(indx)
        foutname$ = finname$
        fin$ = inpath$ + finname$ + inext$
'                             combine path, out-name and extension here:
        fout$ = outpath$ + foutname$ + outext$
'                                   create temp file-names here:
        Print "input filename = "; fin$
        Print "output filename = "; fout$
        Print "Working......"
'
        GOSUB StartProcess
        GOSUB StartDefines
        GOSUB StartExterns
    NEXT indx

    GOTO TheEnd
'
'==========================================================================
'         EXTRACT AND PROCESS INCLUDEs:
'==========================================================================
StartProcess:
'                                PROCESS #INCLUDEs:
'                                ==================
    OPEN "I", #1, fin$
    OPEN "O", #2, tmpfile1$
'
    PRINT #2, "; INCLUDE FILES:"
'
StartLoopA1:
    LINE INPUT#1, input$
'
    IF EOF(1) THEN
        GOTO FinishA1
    ELSEIF input$="" THEN
        GOTO StartLoopA1
    ENDIF
'                                     get length of input string:
'                                     if length is too short, skip it:
    len1 = LEN(input$)
    IF len1 < 2 THEN
        input$ = ""
        GOTO StartLoopA1
    ENDIF
'                            need to look for: #include statements here:
    IF len1 >= 8 THEN
        test1$ = LEFT$(input$, 8)
    ELSE
        input$ = ""
        GOTO StartLoopA1
    ENDIF
'                                if this is a valid #include, verify it:
    IF test1$ = include$ THEN
        pi = 9
        CALL IsWhitespace(pi)
        pi = rval + 1
        CALL getPathFilename(pi, len1)
        test1$ = "; include  "
        len1 = LEN(rstr$)
        len1 = len1 - 1
        ch$ = LEFT$(rstr$, len1)
        test1$ = test1$ + ch$ + "inc"

        PRINT#2, test1$
    ENDIF

    input$ = ""
    GOTO StartLoopA1
'
FinishA1:
    CLOSE 1, 2
'                                rename temp file to permenant filename:
    NAME "tempfile1.tmp", "includes.fil"
    RETURN
'
'==========================================================================
'         EXTRACT AND PROCESS DEFINEs
'==========================================================================
StartDefines:
'                                PROCESS #DEFINES (part-1):
'                                ==========================
    OPEN "I", #1, fin$
    OPEN "O", #2, tmpfile1$
'
StartLoop1:
    LINE INPUT#1, input$
'
    IF EOF(1) THEN
        GOTO Finish1
    ELSEIF input$="" THEN
        GOTO StartLoop1
    ENDIF
'                                     get length of input string:
'                                     if length is too short, skip it:
    len1 = LEN(input$)
    IF len1 < 2 THEN
        input$ = ""
        GOTO StartLoop1
    ENDIF
'                            need to look for: #define statements here:
    IF len1 >= 7 THEN
        test1$ = LEFT$(input$, 7)
    ELSE
        input$ = ""
        GOTO StartLoop1
    ENDIF
'                                if this is a valid #define, verify it:
    ndx = 0
    ch$ = LEFT$(input$, 3)

    IF test1$ = define1$ THEN
        ndx = 8
    ELSEIF test1$ = define2$ THEN
        ndx = 9
    ELSEIF ch$ = define3$ THEN
        pi = 2
        CALL fetchVarName(pi)

        IF rstr$ = define4$ THEN
            ndx = 9
        ENDIF
    ELSE 
        ndx = 0
    ENDIF
'                                        store raw #define to file:
    IF ndx = 8 OR ndx = 9 THEN
        PRINT#2, input$
    ENDIF

    input$ = ""
    GOTO StartLoop1
'
Finish1:
    CLOSE 1, 2
'
'==========================================================================
'==========================================================================
    OPEN "I", #1, tmpfile1$
    OPEN "O", #2, tmpfile2$
    OPEN "O", #4, tmpfile4$
'                                       PROCESS #DEFINES (part-2):
'                                       ==========================
    PRINT #2, "; "
    PRINT #2, ";  EQUATES:"
'
StartLoop2:
    LINE INPUT#1, input$
'
    IF EOF(1) THEN
        GOTO Finish2
    ELSEIF input$="" THEN
        GOTO StartLoop2
    ENDIF

    len1 = LEN(input$)
    CALL StripDefine()

    input$ = ""
    GOTO StartLoop2
'
Finish2:
    CLOSE 1, 4
'
'==========================================================================
'==========================================================================
    OPEN "I", #4, tmpfile4$
'                                       PROCESS #DEFINES (part-3):
'                                       ==========================
StartLoop2b:
    LINE INPUT#4, input$
'
    IF EOF(4) THEN
        GOTO Finish2b
    ELSEIF input$="" THEN
        GOTO StartLoop2b
    ENDIF

    len1 = LEN(input$)
    CALL MakeQuotedStr()

    input$ = ""
    GOTO StartLoop2b
'
Finish2b:
    CLOSE 2, 4
'
    NAME "tempfile2.tmp", "defines.fil"
'
    RETURN

'==========================================================================
'         EXTRACT AND PROCESS EXTERNs:
'==========================================================================
StartExterns:
'                                       PROCESS EXTERNS (part-1):
'                                       =========================
    OPEN "I", #1, fin$
    OPEN "O", #2, tmpfile3$
'
StartLoop3:
    LINE INPUT#1, input$
'
    IF EOF(1) THEN
        GOTO Finish3
    ELSEIF input$="" THEN
        GOTO StartLoop3
    ENDIF
'                                      get length of input string:
'                                      if length is too short, skip it:
    len1 = LEN(input$)
    IF len1 < 2 THEN
        input$ = ""
        GOTO StartLoop3
    ENDIF
'                              need to look for: extern statements here:
    IF len1 >= 6 THEN
        test1$ = LEFT$(input$, 6)
    ELSE
        input$ = ""
        GOTO StartLoop3
    ENDIF
'                                   if this is a valid extern, verify it:
    ndx = 0
    IF test1$ = extern$ THEN
'                               compound multi-line function declarations:
        ndx = 7
        CALL CompExtern(ndx)
    ENDIF

    input$ = ""
    GOTO StartLoop3
'
Finish3:
    CLOSE 1, 2
'==========================================================================
'==========================================================================
'                                       PROCESS EXTERNS (part-2):
'                                       =========================
    OPEN "I", #1, tmpfile3$
    OPEN "O", #2, tmpfile1$
'
StartLoop4:
    LINE INPUT#1, input$
'
    IF EOF(1) THEN
        GOTO Finish4
    ELSEIF input$="" THEN
        GOTO StartLoop4
    ENDIF
'                                    clean out some of the garbage:
    len1 = LEN(input$)
    CALL StripExtern()

    input$ = ""
    GOTO StartLoop4
'
Finish4:
    CLOSE 1, 2
'==========================================================================
'==========================================================================
'
    OPEN "I", #1, tmpfile1$
    OPEN "O", #2, tmpfile3$
'
    PRINT #2, "; "
    PRINT #2, ";  PROTOTYPES:"
'
StartLoop5:
    LINE INPUT#1, input$
'
    IF EOF(1) THEN
        GOTO Finish5
    ELSEIF input$="" THEN
        GOTO StartLoop5
    ENDIF
'                                construct function declaration file:
    len1 = LEN(input$)
    CALL BuildInc()

    input$ = ""
    GOTO StartLoop5
'
Finish5:
    CLOSE 1, 2
'
'==========================================================================
'==========================================================================

    OPEN "I", #4, "includes.fil"
    OPEN "I", #1, "defines.fil"
    OPEN "I", #2, tmpfile3$
    OPEN "O", #3, fout$
'
    input$ = ""

StartLoopA6:
    LINE INPUT#4, input$
'
    IF EOF(4) THEN
        GOTO FinishA6
    ELSEIF input$="" THEN
        GOTO StartLoopA6
    ELSE
        PRINT#3, input$
    ENDIF

    input$ = ""
    GOTO StartLoopA6
'
FinishA6:
    CLOSE 4
'
StartLoop6:
    LINE INPUT#1, input$
'
    IF EOF(1) THEN
        GOTO Finish6
    ELSEIF input$="" THEN
        GOTO StartLoop6
    ELSE
        PRINT#3, input$
    ENDIF

    input$ = ""
    GOTO StartLoop6
'
Finish6:
    CLOSE 1
'
    input$ = ""

StartLoop7:
    LINE INPUT#2, input$
'
    IF EOF(2) THEN
        GOTO Finish7
    ELSEIF input$="" THEN
        GOTO StartLoop7
    ELSE
        PRINT#3, input$
    ENDIF

    input$ = ""
    GOTO StartLoop7
'
Finish7:
    CLOSE 2, 3
'
    Print "Done."
    KILL tmpfile1$
    KILL tmpfile2$
    KILL tmpfile3$
    KILL tmpfile4$
    KILL "defines.fil"
    KILL "includes.fil"
'
    RETURN
'
' ------------------------------------------
TheEnd:
  END
' ------------------------------------------
'==========================================================================
'==========================================================================

'==========================================================================
'==========================================================================
'==========================================================================

' ------------------------------------------
SUB BuildInc()
    DIM new$ = "", tst$ = ",)", padd$ = ""
    DIM chr$ = "", chr2$ = ""
    DIM pi = 1, lenx = len1, len2 = 0, adj = 0
    DIM isvoid = 0, pi2 = 0
'                             this is a rather complex set of processes,
'                             that filters thru the current temporary file
'                             and renders a output file containing 
'                             function prototypes, like:
'                                   "func-name proto :param, :param"
    CALL getVarName2(pi, lenx)
    pi = rval
    len2 = LEN(rstr$)

    IF len2 < 24 THEN
        adj = 24 - len2
    ELSEIF len2 < 30 THEN
        adj = 30 - len2
    ELSEIF len2 < 36 THEN
        adj = 36 - len2
    ELSE
        adj = 40 - len2
    ENDIF

    padd$ = SPACE$(adj)
    new$ = rstr$ + padd$ + "proto"
    chr$ = "("
    rval = 0
    rval = INSTR(input$, chr$)
    pi = rval + 1

    CALL IsWhitespace(pi)
    pi = rval

BuildInc1:
    CALL getVarName(pi, lenx)
    pi = rval
'                                 this could be: char *, void *, fpos_t
'                                 determine which variation:
    IF rstr$ = "__const" THEN
        CALL SortConst(pi, lenx)
        new$ = new$ + rstr$
        pi = rval
'                                      foo(const ...)
    ELSEIF rstr$ = "const" THEN
        CALL SortConst(pi, lenx)
        new$ = new$ + rstr$
        pi = rval
'                                      foo(NCURSES_CONST ...)
    ELSEIF rstr$ = "NCURSES_CONST" THEN
        CALL SortConst(pi, lenx)
        new$ = new$ + rstr$
        pi = rval
'                                      foo(int)
    ELSEIF rstr$ = "int" THEN
        new$ = new$ + " :DWORD"
        CALL IsWhitespace(pi)
        pi = rval
        CALL getVarName(pi, lenx)
        pi = rval
'                                      foo(void *) or: foo(void)
    ELSEIF rstr$ = "void" THEN
        chr$ = MID$(input$, pi, 1)
        pi2 = pi + 1
        chr2$ = MID$(input$, pi2, 1)

        IF chr$ = "*" THEN
            new$ = new$ + " :ptr"
            CALL getVarName(pi, lenx)
            pi = rval
        ELSEIF chr2$ = "*" THEN
            new$ = new$ + " :ptr"
            CALL getVarName(pi2, lenx)
            pi = rval
        ELSE
            isvoid = 1
        ENDIF
'                                      foo(char *)
    ELSEIF rstr$ = "char" THEN
        new$ = new$ + " :ptr SBYTE"
        pi = pi + 2
        CALL getVarName(pi, lenx)
        pi = rval
'                                      foo(FILE *)
    ELSEIF rstr$ = "FILE" THEN
        new$ = new$ + " :ptr FILE"
        pi = pi + 2
        CALL getVarName(pi, lenx)
        pi = rval
'                                      foo(chtype *)
    ELSEIF rstr$ = "chtype" THEN
        pi2 = pi + 1
        chr$ = MID$(input$, pi2, 1)

        IF chr$ = "*" THEN
            new$ = new$ + " :ptr SBYTE"
            pi = pi2
        ELSE
            new$ = new$ + " :DWORD"
        ENDIF
        rval = pi
'                                      foo(size_t)
    ELSEIF rstr$ = "size_t" THEN
        new$ = new$ + " :size_t"
        CALL IsWhitespace(pi)
        pi = rval
        CALL getVarName(pi, lenx)
        pi = rval
'                                      foo(long int) or foo(long,long)
    ELSEIF rstr$ = "long" THEN
        chr$ = MID$(input$, pi, 1)

        IF chr$ = "," OR chr$ = ")" THEN
            new$ = new$ + " :SDWORD"
        ELSE
            new$ = new$ + " :SDWORD"
            pi = pi + 4
            CALL IsWhitespace(pi)
            pi = rval
            CALL getVarName(pi, lenx)
            pi = rval
        ENDIF
'                                      foo(_G_va_list)
    ELSEIF rstr$ = "_G_va_list" THEN
        new$ = new$ + " :va_list"
        CALL IsWhitespace(pi)
        pi = rval
        CALL getVarName(pi, lenx)
        pi = rval
'                                      foo(fpos_t *)
    ELSEIF rstr$ = "fpos_t" THEN
        new$ = new$ + " :ptr fpos_t"
        pi = pi + 2
        CALL getVarName(pi, lenx)
        pi = rval
'                                      foo(struct *)
    ELSEIF rstr$ = "struct" THEN
        new$ = new$ + " :ptr"
        pi = pi + 10
        CALL getVarName(pi, lenx)
        pi = rval
'                                      foo(WINDOW *)
    ELSEIF rstr$ = "WINDOW" THEN
        new$ = new$ + " :ptr"
'                                      foo(SCREEN *)
    ELSEIF rstr$ = "SCREEN" THEN
        new$ = new$ + " :ptr"
'                                      foo(FIELD *)
    ELSEIF rstr$ = "FIELD" THEN
        new$ = new$ + " :ptr"
'                                      foo(FIELDTYPE *)
    ELSEIF rstr$ = "FIELDTYPE" THEN
        new$ = new$ + " :ptr"
'                                      foo(FORM *)
    ELSEIF rstr$ = "FORM" THEN
        new$ = new$ + " :ptr"
'                                      foo(_nc_eventlist *)
    ELSEIF rstr$ = "_nc_eventlist" THEN
        new$ = new$ + " :ptr"
'                                      foo(DIR *)
    ELSEIF rstr$ = "DIR" THEN
        new$ = new$ + " :ptr"
'                                      foo(__gwchar_t *)
    ELSEIF rstr$ = "__gwchar_t" THEN
        new$ = new$ + " :ptr"
'                                      foo(_IO_FILE *)
    ELSEIF rstr$ = "_IO_FILE" THEN
        new$ = new$ + " :ptr"
'                                      foo(uintptr_t *)
    ELSEIF rstr$ = "uintptr_t" THEN
        new$ = new$ + " :ptr"
'                                      foo(ITEM *)
    ELSEIF rstr$ = "ITEM" THEN
        new$ = new$ + " :ptr"
'                                      foo(MENU *)
    ELSEIF rstr$ = "MENU" THEN
        new$ = new$ + " :ptr"
'                                      foo(PANEL *)
    ELSEIF rstr$ = "PANEL" THEN
        new$ = new$ + " :ptr"
'                                      foo(pthread_attr_t *)
    ELSEIF rstr$ = "pthread_attr_t" THEN
        new$ = new$ + " :ptr"
'                                      foo(pthread_mutex_t *)
    ELSEIF rstr$ = "pthread_mutex_t" THEN
        new$ = new$ + " :ptr"
'                                      foo(pthread_mutexattr_t *)
    ELSEIF rstr$ = "pthread_mutexattr_t" THEN
        new$ = new$ + " :ptr"
'                                      foo(pthread_rwlock_t *)
    ELSEIF rstr$ = "pthread_rwlock_t" THEN
        new$ = new$ + " :ptr"
'                                      foo(pthread_cond_t *)
    ELSEIF rstr$ = "pthread_cond_t" THEN
        new$ = new$ + " :ptr"
'                                      foo(pthread_condattr_t *)
    ELSEIF rstr$ = "pthread_condattr_t" THEN
        new$ = new$ + " :ptr"
'                                      foo(pthread_spinlock_t *)
    ELSEIF rstr$ = "pthread_spinlock_t" THEN
        new$ = new$ + " :ptr"
'                                      foo(pthread_barrier_t *)
    ELSEIF rstr$ = "pthread_barrier_t" THEN
        new$ = new$ + " :ptr"
'                                      foo(pthread_barrierattr_t *)
    ELSEIF rstr$ = "pthread_barrierattr_t" THEN
        new$ = new$ + " :ptr"
'                                      foo(...) or: foo(?)
    ELSE
        CALL IsWhitespace(pi)
        pi = rval
        chr$ = MID$(input$, pi, 3)

        IF chr$ = "..." THEN
            new$ = new$ + " :VARARG"
            pi = pi + 3
        ELSEIF rstr$ = "" THEN
            isvoid = 1
        ELSE
            new$ = new$ + " :DWORD"
        ENDIF
    ENDIF
'                                this is a: foo(void)
    IF isvoid = 1 THEN
        GOTO BuildFinish
    ENDIF

    rval = 0
    chr$ = MID$(input$, pi, 1)
    rval = INSTR(tst$, chr$)

    WHILE rval = 0
        pi = pi + 1
        chr$ = MID$(input$, pi, 1)
        rval = INSTR(tst$, chr$)
    WEND

    IF rval = 1 THEN
        new$ = new$ + ","
        pi = pi + 1
        CALL IsWhitespace(pi)
        pi = rval
        GOTO BuildInc1
    ENDIF

BuildFinish:

    input$ = new$
    PRINT#2, input$

BuildFinish2:

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB SortConst(pi, lenx)
    DIM retstr$ = ""
    DIM chr$ = ""
    DIM pi2 = 0
'                               something like: foo(const bar *, ...):
    CALL IsWhitespace(pi)
    pi = rval
    CALL getVarName2(pi, lenx)
    pi = rval
'                                      foo(const char *)
    IF rstr$ = "char" THEN
        retstr$ = " :ptr SBYTE"
        pi = pi + 2
        CALL getVarName(pi, lenx)
'                                      foo(const void *)
    ELSEIF rstr$ = "void" THEN
        retstr$ = " :ptr"
        pi = pi + 2
        CALL getVarName(pi, lenx)
'                                      foo(const fpos_t *)
    ELSEIF rstr$ = "fpos_t" THEN
        retstr$ = " :ptr"
        pi = pi + 2
        CALL getVarName(pi, lenx)
'                                      foo(const fpos64_t *)
    ELSEIF rstr$ = "fpos64_t" THEN
        retstr$ = " :ptr"
        pi = pi + 2
        CALL getVarName(pi, lenx)
'                                      foo(const struct)
    ELSEIF rstr$ = "struct" THEN
        retstr$ = " :ptr"
        CALL IsWhitespace(pi)
        pi = rval
        CALL getVarName(pi, lenx)
'                                      foo(const chtype *)
    ELSEIF rstr$ = "chtype" THEN
        pi2 = pi + 1
        chr$ = MID$(input$, pi2, 1)

        IF chr$ = "*" THEN
            retstr$ = " :ptr SBYTE"
            pi = pi2
        ELSE
            retstr$ = " :DWORD"
        ENDIF
        rval = pi
'                                      foo(const WINDOW *)
    ELSEIF rstr$ = "WINDOW" THEN
        retstr$ = " :ptr"
'                                      foo(const FIELD *)
    ELSEIF rstr$ = "FIELD" THEN
        retstr$ = " :ptr"
'                                      foo(const FORM *)
    ELSEIF rstr$ = "FORM" THEN
        retstr$ = " :ptr"
'                                      foo(const __gwchar_t *)
    ELSEIF rstr$ = "__gwchar_t" THEN
        retstr$ = " :ptr"
'                                      foo(const wchar_t *)
    ELSEIF rstr$ = "wchar_t" THEN
        retstr$ = " :ptr"
'                                      foo(const ITEM *)
    ELSEIF rstr$ = "ITEM" THEN
        retstr$ = " :ptr"
'                                      foo(const MENU *)
    ELSEIF rstr$ = "MENU" THEN
        retstr$ = " :ptr"
'                                      foo(const PANEL *)
    ELSEIF rstr$ = "PANEL" THEN
        retstr$ = " :ptr"
'                                      foo(const pthread_attr_t *)
    ELSEIF rstr$ = "pthread_attr_t" THEN
        retstr$ = " :ptr"
'                                      foo(const pthread_mutexattr_t *)
    ELSEIF rstr$ = "pthread_mutexattr_t" THEN
        retstr$ = " :ptr"
'                                      foo(const pthread_condattr_t *)
    ELSEIF rstr$ = "pthread_condattr_t" THEN
        retstr$ = " :ptr"
'                                      foo(const pthread_barrierattr_t *)
    ELSEIF rstr$ = "pthread_barrierattr_t" THEN
        retstr$ = " :ptr"
'                                      foo(const pthread_rwlockattr_t *)
    ELSEIF rstr$ = "pthread_rwlockattr_t" THEN
        retstr$ = " :ptr"
    ELSE
        retstr$ = " :DWORD"
    ENDIF

' chr$ = MID$(input$, pi, 1)
' Print "rstr$="; rstr$, "chr$="; chr$; "<", "pi="; pi
' Print input$

    rstr$ = retstr$

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB StripExtern()
    DIM new$ = "", chr$ = ""
    DIM pi = 7, lenx = len1
'                            strip token extern and return type
'                            from statement line:
'                               extern void foo (bar, bar);
'                     cut here: !..........!
    CALL IsWhitespace(pi)
    pi = rval
    CALL getVarName2(pi, lenx)
    pi = rval
    CALL SkipNonAlnum(pi)
    pi = rval
'                             test for: extern long int foo(bar):
'                                               skip past "int ":
' Print "input$="; input$
' Print "pi="; pi

    IF rstr$ = "long " THEN
        CALL getVarName(pi, lenx)
        pi = rval
'                         test for: extern long long int foo(bar):
        IF rstr$ = "long" THEN
            CALL IsWhitespace(pi)
            pi = rval
            CALL getVarName(pi, lenx)
            pi = rval
        ENDIF    

        CALL SkipNonAlnum(pi)
        pi = rval
    ENDIF    
'                               need to test here for: __REDIRECT
'                               and NCURSES_SP_NAME:
    CALL getVarName(pi, lenx)
'                                   exit routine if either name found:
    IF rstr$ = "__REDIRECT" THEN
        GOTO StripFinish
    ELSEIF rstr$ = "__REDIRECT_NTH" THEN
        GOTO StripFinish
    ELSEIF rstr$ = "NCURSES_SP_NAME" THEN
        GOTO StripFinish:
    ENDIF

    chr$ = MID$(input$, pi, 1)
    
    WHILE pi <= len1
        new$ = new$ + chr$
        pi = pi + 1
        chr$ = MID$(input$, pi, 1)
    WEND

' Print "new$="; new$
' Print
'                            now test for and eliminate typecasts:
'                            foo (void *(*) (void *, long), void *);
    CALL StripTypecasts(new$)
    PRINT#2, input$

' Print "input$="; input$
' Print

StripFinish:

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB StripTypecasts(sav1$)
    DIM chr$ = ";", newx$ = "", lparens$ = "(", rparens$ = ")"
    DIM semi = 0, rparen = 0, lparen = 0
    DIM pi = 0, xlen = 0
'                            now test for and eliminate typecasts:
'                            foo (struct obstack *, int, int,
'                                 void *(*) (void *, long),
'                                 void (*) (void *, void *), void *);
    semi = INSTR(sav1$, chr$)
    rparen = semi - 1
    chr$ = "("
    lparen = INSTR(sav1$, chr$)
    pi = lparen
    newx$ = LEFT$(sav1$, pi)
    pi = lparen + 1

' Print "sav1$="; sav1$: "newx$="; newx$: "pi="; pi

    WHILE pi < rparen
        chr$ = MID$(sav1$, pi, 1)

        IF chr$ = lparens$ THEN
            WHILE chr$ <> rparens$
                pi = pi + 1
                chr$ = MID$(sav1$, pi, 1)

                IF chr$ = "*" THEN
                    newx$ = newx$ + chr$
                ENDIF
            WEND
        ELSE
            newx$ = newx$ + chr$
        ENDIF
        pi = pi + 1
    WEND
    xlen = LEN(sav1$)

' chr$ = MID$(sav1$, pi, 1)
' Print "newx$="; newx$: "pi="; pi, "chr$="; chr$, "xlen="; xlen

    WHILE pi <= xlen
        chr$ = MID$(sav1$, pi, 1)
        newx$ = newx$ + chr$
        pi = pi + 1
    WEND
    input$ = newx$

' Print "input$="; input$

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB CompExtern(indx)
    DIM chr$ = "", new$ = "", chr1$ = ");"
    DIM chr2$ = "__THROW", chr3$ = "__nonnull"
    DIM chr4$ = "__attribute", chr5$ = "__MALLOC_DEPRECATED"
    DIM chr6$ = "__cleanup_fct_attribute", chr7$ = "__wur"
    DIM chr8$ = "GCC_PRINTFLIKE"
    DIM flag = 0, ff1 = 0, ff2 = 0, ff3 = 0
    DIM ff4 = 0, ff5 = 0, ff6 = 0, ff7 = 0, ff8 = 0

'                         this function compounds multi-line
'                         function statements into a single line:
'
'                        begin by determining the function return type:
    CALL ExternType(indx)

    IF rval = 0 THEN
        GOTO CompFinish
    ELSE
        new$ = input$
        input$ = ""
    ENDIF

    chr$ = "("
    rval = 0
    rval = INSTR(new$, chr$)
'                       in the event that there is not an "(", then, 
'                       this would mean that this is not a function:
'                                      extern void blah blah blah:
    IF rval = 0 THEN
        GOTO CompFinish
    ENDIF
'                       this portion contains a loop that collects
'                       additional function fragments that reside
'                       on multiple lines:
CompLoop1:
'    rval = 0
'    rval = INSTR(new$, chr1$)

    ff1 = INSTR(new$, chr1$)
    ff2 = INSTR(new$, chr2$)
    ff3 = INSTR(new$, chr3$)
    ff4 = INSTR(new$, chr4$)
    ff5 = INSTR(new$, chr5$)
    ff6 = INSTR(new$, chr6$)
    ff7 = INSTR(new$, chr7$)
    ff8 = INSTR(new$, chr8$)

    flag = ff1 + ff2 + ff3 + ff4 + ff5 + ff6 + ff7 + ff8

' Print "flag="; flag
'    IF rval = 0 THEN
    IF flag = 0 THEN
        LINE INPUT#1, input$

        IF EOF(1) THEN
            GOTO CompFinish
        ELSEIF input$="" THEN
            GOTO CompLoop1
        ENDIF
        new$ = new$ + input$
        input$ = ""
        GOTO CompLoop1
    ENDIF

    input$ = new$
    PRINT#2, input$

CompFinish:

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB ExternType(xndx)
    DIM chr$ = "", savstr$ = ""
    DIM parenstar$ = "(*", starparenstar$ = "*(*"
    DIM pi = xndx, lenx = len1, sav1 = 0
    DIM pspi = 0, sppi = 0
'                            this function identifies valid function 
'                            types, based on the return type:
    CALL IsWhitespace(pi)
    pi = rval
    CALL getVarName2(pi, lenx)
    pi = rval
' Print "rstr$="; rstr$

    IF rstr$ = "NCURSES_EXPORT" THEN
'                     here, test for: NCURSES_EXPORT(int) foo(bar)
        CALL fixNCursesExport()

    ELSEIF rstr$ = "struct" THEN
'                     here, test for: struct name * foo(bar)
        sav1 = pi
        CALL fixStruct()
        pi = sav1

    ELSEIF rstr$ = "const" THEN
'                     here, test for: const char * foo(bar)
        CALL fixConstChar()

    ELSEIF rstr$ = "__const" THEN
'                     here, test for: __const char * foo(bar)
        CALL fixConstChar2()

    ELSEIF rstr$ = "unsigned" THEN
'                     here, test for: unsigned long int foo(bar)
        CALL fixUnsigned()

' Print "rstr$="; rstr$
' Print input$
' Print

    ELSEIF rstr$ = "enum" THEN
'                     here, test for: enum mcheck_status foo(bar)
        CALL fixEnum()
    ENDIF

    CALL IsWhitespace(pi)
    pi = rval
    chr$ = MID$(input$, pi, 1)

' Print "rstr$="; rstr$, "pi="; pi, "chr$="; chr$
' Print input$
    pspi = INSTR(input$, parenstar$)
    sppi = INSTR(input$, starparenstar$)

    IF pspi = pi OR sppi = pi THEN
'                      here we are trapping some weird function pointers:
'                             extern void (*error_print_progname) (void);
        savstr$ = rstr$
        CALL fixFuncPntr(pi)
        rstr$ = savstr$
        chr$ = MID$(input$, pi, 1)
    ENDIF

    CALL GetAlpha(chr$)

    IF rval = 0 AND chr$ <> "*" THEN
        rval = 0
        GOTO ExtTypeFinish
    ENDIF
    rval = 0

' Print "rstr$="; rstr$
' Print input$

    IF rstr$ = "FILE" THEN
        rval = 1
    ELSEIF rstr$ = "DIR" THEN
        rval = 1
    ELSEIF rstr$ = "ENTRY" THEN
        rval = 1
    ELSEIF rstr$ = "WINDOW" THEN
        rval = 1
    ELSEIF rstr$ = "__FILE" THEN
        rval = 1
    ELSEIF rstr$ = "NCURSES_EXPORT" THEN
        rval = 1
    ELSEIF rstr$ = "int" THEN
        rval = 1
    ELSEIF rstr$ = "char" THEN
        rval = 1
    ELSEIF rstr$ = "void" THEN
        rval = 1
    ELSEIF rstr$ = "long" THEN
        rval = 1
    ELSEIF rstr$ = "enum" THEN
        rval = 1
    ELSEIF rstr$ = "bool" THEN
        rval = 1
    ELSEIF rstr$ = "float" THEN
        rval = 1
    ELSEIF rstr$ = "const" THEN
        rval = 1
    ELSEIF rstr$ = "struct" THEN
        rval = 1
    ELSEIF rstr$ = "chtype" THEN
        rval = 1
    ELSEIF rstr$ = "double" THEN
        rval = 1
    ELSEIF rstr$ = "unsigned" THEN
        rval = 1
    ELSEIF rstr$ = "mseRegion" THEN
        rval = 1
    ELSEIF rstr$ = "mqd_t" THEN
        rval = 1
    ELSEIF rstr$ = "sem_t" THEN
        rval = 1
    ELSEIF rstr$ = "div_t" THEN
        rval = 1
    ELSEIF rstr$ = "ldiv_t" THEN
        rval = 1
    ELSEIF rstr$ = "size_t" THEN
        rval = 1
    ELSEIF rstr$ = "time_t" THEN
        rval = 1
    ELSEIF rstr$ = "wint_t" THEN
        rval = 1
    ELSEIF rstr$ = "ssize_t" THEN
        rval = 1
    ELSEIF rstr$ = "error_t" THEN
        rval = 1
    ELSEIF rstr$ = "iconv_t" THEN
        rval = 1
    ELSEIF rstr$ = "speed_t" THEN
        rval = 1
    ELSEIF rstr$ = "clock_t" THEN
        rval = 1
    ELSEIF rstr$ = "nl_catd" THEN
        rval = 1
    ELSEIF rstr$ = "wchar_t" THEN
        rval = 1
    ELSEIF rstr$ = "wctype_t" THEN
        rval = 1
    ELSEIF rstr$ = "td_err_e" THEN
        rval = 1
    ELSEIF rstr$ = "intmax_t" THEN
        rval = 1
    ELSEIF rstr$ = "po_file_t" THEN
        rval = 1
    ELSEIF rstr$ = "imaxdiv_t" THEN
        rval = 1
    ELSEIF rstr$ = "uintmax_t" THEN
        rval = 1
    ELSEIF rstr$ = "uintptr_t" THEN
        rval = 1
    ELSEIF rstr$ = "pthread_t" THEN
        rval = 1
    ELSEIF rstr$ = "wctrans_t" THEN
        rval = 1
    ELSEIF rstr$ = "reg_syntax_t" THEN
        rval = 1
    ELSEIF rstr$ = "po_message_t" THEN
        rval = 1
    ELSEIF rstr$ = "po_filepos_t" THEN
        rval = 1
    ELSEIF rstr$ = "po_message_iterator_t" THEN
        rval = 1
    ELSEIF rstr$ = "__off_t" THEN
        rval = 1
    ELSEIF rstr$ = "__uid_t" THEN
        rval = 1
    ELSEIF rstr$ = "__gid_t" THEN
        rval = 1
    ELSEIF rstr$ = "__pid_t" THEN
        rval = 1
    ELSEIF rstr$ = "__off64_t" THEN
        rval = 1
    ELSEIF rstr$ = "__ssize_t" THEN
        rval = 1
    ELSEIF rstr$ = "__locale_t" THEN
        rval = 1
    ELSEIF rstr$ = "__useconds_t" THEN
        rval = 1
    ELSEIF rstr$ = "__sighandler_t" THEN
        rval = 1
    ELSEIF rstr$ = "_IO_size_t" THEN
        rval = 1
    ELSEIF rstr$ = "_IO_wint_t" THEN
        rval = 1
    ELSEIF rstr$ = "_IO_ssize_t" THEN
        rval = 1
    ELSEIF rstr$ = "_IO_off64_t" THEN
        rval = 1

    ENDIF    

ExtTypeFinish:

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB fixFuncPntr(indx)
    DIM new$ = "", lparen$ = ")"
    DIM pi = 0, lpi = 0, lena = 0, lenb = 0
'                             extern void (*error_print_progname) (void);
    pi = indx - 1
    new$ = LEFT$(input$, pi)
' Print "new$="; new$; "<"

    lpi = INSTR(input$, lparen$)
    lpi = lpi - 1
    CALL WalkBackward(lpi)
    pi = rval + 1
' Print "pi="; pi

    lena = LEN(input$)
    CALL getVarName2(pi, lena)
    pi = rval
    new$ = new$ + rstr$
' Print "new$="; new$; "<"

    lenb = lena - pi
    new$ = new$ + RIGHT$(input$, lenb)
' Print "new$="; new$; "<"

    input$ = new$

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB fixEnum()
    DIM new$ = "", test1$ = ""
    DIM pstr$ = "(", mstr$ = "mcheck_status"
    DIM pi = 0, ppi = 0, mpi = 0
    DIM lena = 0, lenb = 0

'                    this sub filters out: extern enum mcheck_status:
'                     example: 
'                                  extern enum mcheck_status foo(bar):
'                         cut here:............!------------!
    new$ = LEFT$(input$, 12)
' Print "new$="; new$; "<"

    pi = 12
    CALL IsWhitespace(pi)
    pi = rval
' Print "pi="; pi

    ppi = INSTR(input$, pstr$)
    mpi = INSTR(input$, mstr$)
' Print "ppi="; ppi, "ipi="; ipi, "lpi="; lpi

    IF mpi < ppi THEN
        lena = LEN(input$)
        CALL getVarName(mpi, lena)

        pi = rval
        CALL IsWhitespace(pi)
        pi = rval - 1
        lenb = lena - pi
        test1$ = RIGHT$(input$, lenb)
' Print "pi="; pi, "test1$="; test1$

        new$ = new$ + test1$
' Print "new$="; new$; "<"
' Print

    ENDIF

    input$ = new$
    rstr$ = "enum"
    rval = pi

ENDSUB
' ------------------------------------------



' ------------------------------------------
SUB fixUnsigned()
    DIM new$ = "", test1$ = ""
    DIM pstr$ = "(", istr$ = "int", lstr$ = "long"
    DIM pi = 0, ppi = 0, ipi = 0, lpi = 0
    DIM lena = 0, lenb = 0

'                     this sub filters out: extern unsigned long int:
'                                      extern unsigned long long int:
'                     example: 
'                                  extern unsigned long int foo(bar):
'                         cut here:................!-------!
    new$ = LEFT$(input$, 16)
' Print "new$="; new$; "<"

    pi = 16
    CALL IsWhitespace(pi)
    pi = rval
' Print "pi="; pi

    ppi = INSTR(input$, pstr$)
    ipi = INSTR(input$, istr$)
    lpi = INSTR(input$, lstr$)
' Print "ppi="; ppi, "ipi="; ipi, "lpi="; lpi

    IF ipi < ppi OR lpi < ppi THEN
        lena = LEN(input$)

        IF ipi > lpi THEN
' Print "int is greater"

            CALL getVarName(ipi, lena)
        ELSEIF lpi > ipi THEN
' Print "long is greater"

            CALL getVarName(lpi, lena)
        ENDIF

        pi = rval
        CALL IsWhitespace(pi)
        pi = rval - 1
        lenb = lena - pi
        test1$ = RIGHT$(input$, lenb)
' Print "pi="; pi, "test1$="; test1$

        new$ = new$ + test1$
' Print "new$="; new$; "<"
' Print

    ENDIF

    input$ = new$
    rstr$ = "unsigned"
    rval = pi

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB fixConstChar()
    DIM chr$ = "", new$ = "", test1$ = ""
    DIM astr$ = "*", pstr$ = "("
    DIM pi = 0, lena = 0, lenb = 0
    DIM aster = 0, paren = 0
'                              this sub filters out const char *:
'                example: 
'                          extern const char * const * foo(bar);
'                 cut here:.............!-------------!

    new$ = MID$(input$, 1, 13)
    pi = 14

    CALL IsWhitespace(pi)
    pi = rval
    lena = LEN(input$)
    CALL getVarName(pi, lena)
    pi = rval

    IF rstr$ = "char" THEN
        CALL IsWhitespace(pi)
'        pi = rval + 1
        pi = rval
        lenb = lena - pi
        test1$ = RIGHT$(input$, lenb)
' Print "test1$="; test1$

        aster = INSTR(test1$, astr$)
        paren = INSTR(test1$, pstr$)

        IF aster > 0 AND aster < paren THEN
            pi = aster + 1
            lena = LEN(test1$)
            lenb = lena - pi
            new$ = new$ + RIGHT$(test1$, lenb)
        ELSE
            new$ = new$ + test1$
        ENDIF
    ENDIF

    input$ = new$
    rstr$ = "const"
    rval = pi

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB fixConstChar2()
    DIM chr$ = "", new$ = "", test1$ = ""
    DIM astr$ = "*", pstr$ = "("
    DIM pi = 0, lena = 0, lenb = 0
    DIM aster = 0, paren = 0
'                              this sub filters out __const char *:
'                example: 
'                              extern __const char *foo (bar);
'                     cut here:...............!----!

    new$ = MID$(input$, 1, 15)
    pi = 16

    CALL IsWhitespace(pi)
    pi = rval
    lena = LEN(input$)
    CALL getVarName(pi, lena)
    pi = rval

    IF rstr$ = "char" THEN
        CALL IsWhitespace(pi)
        pi = rval

        lenb = lena - pi
        test1$ = RIGHT$(input$, lenb)

        aster = INSTR(test1$, astr$)
        paren = INSTR(test1$, pstr$)

        IF aster > 0 AND aster < paren THEN
            pi = aster + 1
            CALL IsWhitespace(pi)
            pi = rval
            lena = LEN(test1$)
            lenb = lena - pi
            new$ = new$ + RIGHT$(test1$, lenb)
        ELSE
            new$ = new$ + test1$
        ENDIF
    ENDIF

    input$ = new$
    rstr$ = "const"
    rval = pi

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB fixStruct()
    DIM chr$ = "", new$ = ""
    DIM pi = 0, lena = 0, lenb = 0
'                              this sub filters out the struct-name
'                example: 
'                        extern struct aliasent *getaliasent (void);
'                cut here:.............!--------!
    new$ = MID$(input$, 1, 14)
    pi = 14
    CALL IsWhitespace(pi)
    pi = rval
    lena = LEN(input$)
    CALL skipVarName(pi, lena)
    pi = rval
'                                this call destroys rstr$:
    CALL SkipNonAlnum(pi)
    pi = rval - 1

    lena = LEN(input$)
    lenb = lena - pi
    new$ = new$ + RIGHT$(input$, lenb)

    input$ = new$
    rstr$ = "struct"

' Print "input$="; input$

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB fixNCursesExport()
    DIM chr$ = "", new$ = ""
    DIM pi = 0, lena = 0, lenb = 0
'                              this sub filters out a portion of ncurses 
'                              extern function declarations:
    new$ = MID$(input$, 1, 21)
    pi = 21
    chr$ = MID$(input$, pi, 1)

    WHILE chr$ <> ")"
        pi = pi + 1
        chr$ = MID$(input$, pi, 1)
    WEND

'    pi = pi + 1
    lena = LEN(input$)
    lenb = lena - pi
'    new$ = new$ + MID$(input$, pi, lenb)
    new$ = new$ + RIGHT$(input$, lenb)
    input$ = new$
' Print "input$="; input$

ENDSUB
' ------------------------------------------

'=====================================================================
'=====================================================================

' ------------------------------------------
SUB StripDefine()
    DIM new$ = "", chr$ = "define"
    DIM pi = 0, lenb = 0
'                        this function prepares #define statements for
'                        stripping of non-essential verbage and storage:
' Print "input$="; input$ : "len1="; len1
'                                  locate token "define":
    pi = INSTR(input$, chr$)
'                            advance to next token, example:
'                            #define _G_IO_IO_FILE_VERSION 0x20001
'                           ->.......!
' Print "pi="; pi
    pi = pi + 6

' Print "pi="; pi
    CALL IsWhitespace(pi)
    pi = rval - 1

' Print "pi="; pi
'                              now, store everything to the rignt:
    lenb = len1 - pi
    new$ = RIGHT$(input$, lenb)
    input$ = new$
' Print "new$="; new$

    CALL StripComments()
' Print input$; "<"
'                        test for possible problem defines, like macros:
'                        get length of input string:
    CALL VerifyInput()
    len1 = LEN(input$)

    IF len1 > 0 THEN
        CALL MakeEquate()
        PRINT#2, input$
    ENDIF

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB MakeEquate()
    DIM new$ = "", chr$ = "", padd$ = ""
    DIM pi = 1, len2 = 0, adj = 0
'                       could add an: ifndef foobar
'                       here:
'                             get first parameter:
    chr$ = MID$(input$, pi, 1)
    CALL GetAlNum(chr$)
    
    WHILE rval > 0
        new$ = new$ + chr$
        pi = pi + 1
        chr$ = MID$(input$, pi, 1)
        CALL GetAlNum(chr$)
    WEND
'                           if this is a function, abort process:
    IF chr$ = "(" THEN
        new$ = ";"
        GOTO EndMakeEqu
    ENDIF
'                           add space padding after var-name
    len2 = LEN(new$)
    IF len2 < 24 THEN
        adj = 24 - len2
    ELSEIF len2 < 30 THEN
        adj = 30 - len2
    ENDIF

    padd$ = SPACE$(adj)
    new$ = new$ + padd$ + "EQU" + CHR$(9)

    IF pi >= len1 THEN
        new$ = new$ + "<>"
    ELSE
        CALL IsWhitespace(pi)
        pi = rval
'                             get second parameter
        chr$ = MID$(input$, pi, 1)
        CALL GetAlNum(chr$)
    
        WHILE rval > 0
            new$ = new$ + chr$
            pi = pi + 1
            chr$ = MID$(input$, pi, 1)
            CALL GetAlNum(chr$)
        WEND
    ENDIF    

EndMakeEqu:
'**    input$ = new$
    input$ = ";" + new$

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB VerifyInput()
    DIM testa$ = "define"
    DIM testb$ = "("
    DIM testc$ = ""
    DIM testd$ = "\"
    DIM teste$ = ""
    DIM testf$ = "{"
    DIM testg$ = "0x"
    DIM new$ = ""
    DIM pix = 0, lenx = 0, sav1 = 0, lenb = 0

    testc$ = CHR$(34)
    teste$ = CHR$(39)
'                          test for problematic defines including
'                           macros and multi-line definitions and
'                                                 quoted strings:
'
'                                  this is for a buried "define":
    rval = 0
    rval = INSTR(input$, testa$)
    IF rval > 0 THEN
        input$ = ""
        GOTO EndVerify
    ENDIF
'                                          this is a macro paren:
    rval = 0
    rval = INSTR(input$, testb$)
    IF rval > 0 THEN
        input$ = ""
        GOTO EndVerify
    ENDIF
'                                         this is a double quote:
    rval = 0
    rval = INSTR(input$, testc$)
    IF rval > 0 THEN
'                          HERE: insert a call to a conversion routine,
'                             that will build a seperate zstr file:
        PRINT#4, input$

        input$ = ""
        GOTO EndVerify
    ENDIF
'                                           this is a back-slash:
    rval = 0
    rval = INSTR(input$, testd$)
    IF rval > 0 THEN
        input$ = ""
        GOTO EndVerify
    ENDIF
'                                         this is a single quote:
    rval = 0
    rval = INSTR(input$, teste$)
    IF rval > 0 THEN
        input$ = ""
        GOTO EndVerify
    ENDIF
'                                          this is a curly brace:
    rval = 0
    rval = INSTR(input$, testf$)
    IF rval > 0 THEN
        input$ = ""
        GOTO EndVerify
    ENDIF
'                                           is this a hex number:
    rval = 0
    rval = INSTR(input$, testg$)
' Print "rval="; rval

    IF rval > 0 THEN
        sav1 = rval
'                         now advance to start of second varname:
        pix = 1
        lenx = LEN(input$)
        CALL getVarName2(pix, lenx)
        pix = rval
        CALL IsWhitespace(pix)
        pix = rval
'                           does location found match 2nd varname:
        IF pix = sav1 THEN
            pix = sav1 - 1
            new$ = LEFT$(input$, pix)
            pix = pix + 2
            lenb = lenx - pix
            testg$ = RIGHT$(input$, lenb)
            CALL TrailingWhitespace(lenb, testg$)
            pix = rval
            testg$ = LEFT$(testg$, pix)
            new$ = new$ + testg$ + "h"
            input$ = new$
        ENDIF
        GOTO EndVerify
    ENDIF

EndVerify:

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB StripComments()
    DIM testa$ = "/*"
    DIM new$ = ""
    DIM pidx = 0, lenqq = 0, sav1 = 0
'                              this utility will strip-out trailing 
'                              comments from a line of code:
'                   example: 
'                           ARG_USAGE   0x01 /* a Usage: message. */
'                    cut here: .............!----------------------!
    rval = 0
    rval = INSTR(input$, testa$)
    IF rval > 0 THEN
        pidx = rval - 1
        new$ = LEFT$(input$, pidx)
        input$ = new$
    ENDIF
'                             test here if this is an empty definition:
'                              i.e.:    #define TEST     /* nothing */
    pidx = 1
    lenqq = LEN(input$)
    CALL getVarName2(pidx, lenqq)
    sav1 = rval
    CALL IsWhitespace(sav1)
    pidx = rval

    IF pidx > lenqq THEN
        pidx = sav1 - 1
        new$ = LEFT$(input$, pidx)
        input$ = new$
    ENDIF

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB MakeQuotedStr()
'**    DIM new$ = "", padd$ = ""
    DIM new$ = ";", padd$ = ""
    DIM pix = 1, lenx = 0, lenx2 = 0, adj = 0
'                     transform quoted string to zero-terminated string:
'                               MNTTYPE_IGNORE    "ignore"	
'                               MNTTYPE_IGNORE          DB  'ignore',0
    lenx = LEN(input$)
    CALL getVarName2(pix, lenx)
'**    new$ = rstr$
    new$ = new$ + rstr$
    pix = rval
    lenx2 = LEN(new$)

    IF lenx2 < 24 THEN
        adj = 24 - lenx2
    ELSEIF lenx2 < 30 THEN
        adj = 30 - lenx2
    ENDIF

    padd$ = SPACE$(adj)
    new$ = new$ + padd$ + "DB" + CHR$(9) + "'"

    CALL IsWhitespace(pix)
    pix = rval + 1
    CALL getQuotedStr(pix, lenx)
    new$ = new$ + rstr$ + "',0"

    PRINT#2, new$

ENDSUB
' ------------------------------------------


'==========================================================================
'==========================================================================

' ------------------------------------------
SUB GetAlpha(chr$)
    DIM underscore$ = "_"
    DIM allcaps$ = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    DIM lower$ = "abcdefghijklmnopqrstuvwxyz"
    DIM hyphen$ = "-"
    DIM testz$ = underscore$ + allcaps$ + lower$ + hyphen$

    rval = 0
    rval = INSTR(testz$, chr$)

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB GetAlNum(chr$)
    DIM underscore$ = "_"
    DIM allcaps$ = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    DIM lower$ = "abcdefghijklmnopqrstuvwxyz"
    DIM nums$ = "0123456789"
    DIM hyphen$ = "-+."
    DIM testz$ = underscore$ + allcaps$ + lower$ + nums$ + hyphen$

    rval = 0
    rval = INSTR(testz$, chr$)

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB GetPath(chr$)
    DIM underscore$ = "_"
    DIM allcaps$ = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    DIM lower$ = "abcdefghijklmnopqrstuvwxyz"
    DIM nums$ = "0123456789"
    DIM hyphen$ = "-+./"
    DIM testz$ = underscore$ + allcaps$ + lower$ + nums$ + hyphen$

    rval = 0
    rval = INSTR(testz$, chr$)

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB IsWhitespace(indx)
    DIM chr$ = ""
    DIM chx = 0
    
    chr$ = MID$(input$, indx, 1)
    chx = ASC(chr$)

    WHILE chx = 32 OR chx = 9
        indx = indx + 1
        chr$ = MID$(input$, indx, 1)
        chx = ASC(chr$)
    WEND

    rval = indx
ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB getVarName(ndx, lenz)
    DIM chr$ = ""
    DIM pi = ndx, lenx = lenz

    rstr$ = ""
    chr$ = MID$(input$, pi, 1)
    CALL GetAlpha(chr$)

    WHILE rval > 0 AND pi < lenx
        rstr$ = rstr$ + chr$
        IF pi < lenx THEN
            pi = pi + 1
            chr$ = MID$(input$, pi, 1)
            CALL GetAlpha(chr$)
        ELSE
            rval = 0
        ENDIF
    WEND
'                 return: name in rstr$, pi in rval
    rval = pi

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB getVarName2(ndx, lenz)
    DIM chr$ = ""
    DIM pi = ndx, lenx = lenz

    rstr$ = ""
    chr$ = MID$(input$, pi, 1)
    CALL GetAlNum(chr$)

    WHILE rval > 0 AND pi < lenx
        rstr$ = rstr$ + chr$
        IF pi < lenx THEN
            pi = pi + 1
            chr$ = MID$(input$, pi, 1)
            CALL GetAlNum(chr$)
        ELSE
            rval = 0
        ENDIF
    WEND
'                 return: name in rstr$, pi in rval
    rval = pi

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB getPathFilename(ndx, lenz)
    DIM chr$ = ""
    DIM pi = ndx, lenx = lenz

    rstr$ = ""
    chr$ = MID$(input$, pi, 1)
    CALL GetPath(chr$)

    WHILE rval > 0 AND pi < lenx
        rstr$ = rstr$ + chr$
        IF pi < lenx THEN
            pi = pi + 1
            chr$ = MID$(input$, pi, 1)
            CALL GetPath(chr$)
        ELSE
            rval = 0
        ENDIF
    WEND
'                 return: name in rstr$, pi in rval
    rval = pi

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB SkipNonAlnum(ndx)
    DIM chr$ = ""
    DIM pi = ndx
'                             skip all non alpha-num chars
    chr$ = MID$(input$, pi, 1)
    CALL GetAlpha(chr$)

    WHILE rval = 0
        rstr$ = rstr$ + chr$
        pi = pi + 1
        chr$ = MID$(input$, pi, 1)
        CALL GetAlpha(chr$)
    WEND
'                 return: name in rstr$, pi in rval
    rval = pi

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB skipVarName(ndx, lenz)
    DIM chr$ = "", test1$ = ""
    DIM pi = ndx, lenx = lenz
'                          this function simply skips over a token name:

    chr$ = MID$(input$, pi, 1)
    CALL GetAlpha(chr$)

    WHILE rval > 0 AND pi < lenx
        test1$ = test1$ + chr$
        IF pi < lenx THEN
            pi = pi + 1
            chr$ = MID$(input$, pi, 1)
            CALL GetAlpha(chr$)
        ELSE
            rval = 0
        ENDIF
    WEND
'                 return: pi in rval, trash varname
    rval = pi

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB TrailingWhitespace(indx, myStr$)
    DIM chr$ = ""
    DIM chx = 0
    
    chr$ = MID$(myStr$, indx, 1)
    chx = ASC(chr$)

    WHILE chx = 32 OR chx = 9
        indx = indx - 1
        chr$ = MID$(myStr$, indx, 1)
        chx = ASC(chr$)
    WEND

    rval = indx
ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB WalkBackward(indx)
    DIM chr$ = ""
'                             extern void (*error_print_progname) (void);
'                     walk back to here:    ^<<<.....<<<.....<<<^    
    chr$ = MID$(input$, indx, 1)
    CALL GetAlNum(chr$)

    WHILE rval <> 0
        indx = indx - 1
        chr$ = MID$(input$, indx, 1)
        CALL GetAlNum(chr$)
    WEND

    rval = indx
ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB fetchVarName(ptr)
    DIM ndx = 0, lenz = 0
'                           ths function is designed to skip over any 
'                               white-spaces and collect the varname:
    CALL IsWhitespace(ptr)
    ndx = rval
    lenz = LEN(input$)
    CALL getVarName(ndx, lenz)
'           return with varname in rstr$:
'           return with pi in rval:

ENDSUB
' ------------------------------------------


' ------------------------------------------
SUB getQuotedStr(ndx, lenz)
    DIM chr$ = "", quote$ = ""
    DIM pi = ndx, lenx = lenz

    quote$ = CHR$(34)
    rstr$ = ""
    chr$ = MID$(input$, pi, 1)

    WHILE chr$ <> quote$ AND pi < lenx
        rstr$ = rstr$ + chr$
        IF pi < lenx THEN
            pi = pi + 1
            chr$ = MID$(input$, pi, 1)
        ENDIF
    WEND
'                 return: string in rstr$, pi in rval
    rval = pi

ENDSUB
' ------------------------------------------






'==========================================================================
'==========================================================================



' TODO LIST:
'============
'==========================================================================
' Struct's: translation of structures can be illustrated by the ".inc" 
' files produced by h2incx.exe. Here is an example from a.out.h:
' struct exec
' {                            ' exec	struct 
'   unsigned long a_info;      ' a_info	DWORD	?
'   unsigned int a_text;       ' a_text	DWORD	?
'   unsigned int a_data;       ' a_data	DWORD	?
'   unsigned int a_bss;        ' a_bss	DWORD	?
'   unsigned int a_syms;       ' a_syms	DWORD	?
'   unsigned int a_entry;      ' a_entry	DWORD	?
'   unsigned int a_trsize;     ' a_trsize	DWORD	?
'   unsigned int a_drsize;     ' a_drsize	DWORD	?
' };                           ' exec	ends
'
' Here is another (different) example from _G_config.h:
' typedef struct
' {                            ' _G_fpos_t	struct 
'   __off_t __pos;             ' __pos       __off_t      ?
'   __mbstate_t __state;       ' __state     __mbstate_t  ?
' } _G_fpos_t;                 ' _G_fpos_t	ends
'
'
'==========================================================================
' Union's: can have a similar appearance to structs, 
' as seen here:
' typedef union
' {                                     ' _G_iconv_t	union 
'   struct __gconv_info __cd;           ' __cd     __gconv_info      <>
'   struct                              ' struct   __combined
'   {
'     struct __gconv_info __cd;         ' __cd     __gconv_info      <>
'     struct __gconv_step_data __data;  ' __data   __gconv_step_data <>
'   } __combined;                       ' ends
' } _G_iconv_t;                         ' _G_iconv_t	ends
'
'
'==========================================================================
' Macro's: are buried in #define statements and have a general appearance 
' of complex looking function declarations. 
' Here is an example from a.out.h:
'
' #define N_MAGIC(exec)	((exec).a_info & 0xffff)
'
' N_MAGIC macro exec
'	exitm <( ( exec ) .a_info  and  0ffffh ) >
'	endm
'
' Macros require careful parsing, as they can often occupy multiple lines.
'
'
